<?php

namespace Drupal\mcapi_credcom_spoof;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Extension\ModuleHandler;
use Symfony\Component\DependencyInjection\ContainerInterface;


class SettingsForm extends ConfigFormBase {

  private $moduleHandler;
  private $traded;

  /**
   * {@inheritdoc}
   */
  public function getFormID() {
    return 'mcapi_credcom_spoof_settings_form';
  }

  /**
   *
   * @param ConfigFactoryInterface $configFactory
   * @param ModuleHandler $module_handler
   * @param QueryInterface $transaction_entity_query
   */
  public function __construct(ConfigFactoryInterface $configFactory, ModuleHandler $module_handler, QueryInterface $transaction_entity_query) {
    $this->setConfigFactory($configFactory);
    $this->moduleHandler = $module_handler;
    $this->traded = (bool)$transaction_entity_query->accessCheck(FALSE)->condition('type', 'remote')->execute();
  }

  static public function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('module_handler'),
      $container->get('entity_type.manager')->getStorage('mcapi_transaction')->getQuery()
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('mcapi_credcom_spoof.settings');

    $form['curr_id'] = [
      '#title' => $this->t('Currency used for intertrading'),
      '#type' => 'mcapi_currency_select',
      '#multiple' => FALSE,
      '#default_value' => $config->get('curr_id'),
      '#weight' => 0,
      '#disabled'=> $this->traded,
      '#required' => TRUE
    ];
    $form['rate'] = [
      '#title' => $this->t('Conversion rate'),
      '#description' => $this->t('The number of (raw) local units to one unit on the parent ledger. Note that local units are stored as integers of the smallest subdivision, e.g. cents not dollars, minutes, not hours.'),
      '#type' => 'number',
      '#min' => 0,
      '#step' => 0.01,
      '#default_value' => $config->get('rate'),
      '#disabled'=> $this->traded,
      '#weight' => 1,
    ];
    $form['wallet_name'] = [
      '#title' => $this->t('Friendly name of the intertrading wallet'),
      '#type' => 'textfield',
      '#default_value' => $config->get('wallet_name'),
      '#weight' => 2,
      '#required' => TRUE
    ];
    $url = $config->get('trunkwards_node_url');
    if ($url and $ccuser = $config->get('cc_user')) {
      // things should only be disabled if there is a transaction history.
      $connected = mcapi_credcom_handshake($url, $ccuser, (string)$config->get('cc_auth'));
    }

    $form['connection'] = [
      '#title' => $this->t('Connection to trunkwards node'),
      '#type' => 'details',
      '#open' => !isset($connected),
      '#weight' => 4,
      'trunkwards_node_url' => [
        '#title' => $this->t('Trunkwards node URL'),
        '#description' => t('Must be https to protect the login credentials'),
        '#type' => 'textfield',
        '#default_value' => $url ?? $this->defaultSiteName(),
        '#placeholder' => 'http://',
        //'#pattern' => 'https:\/\/',
        '#disabled'=> isset($connected),
        '#required' => TRUE,
        '#weight' => 1
      ],
      'cc_user' => [
        '#title' => $this->t('The name of the current site on the network'),
        '#description' => $config->get('cc_user') ? t('This cannot be changed.') : t('The account must exist already on the trunkwards node.'),
        '#type' => 'textfield',
        '#default_value' => $config->get('cc_user'),
        '#placeholder' => $this->t('lowercase with _ and no spaces.'),
        '#pattern' => '[a-z1-9_]+',
        '#weight' => 2,
        '#disabled'=> isset($connected),
        '#required' => TRUE
      ],
      'cc_auth' => [
        '#title' => $this->t('Auth string'),
        '#type' => 'textfield',
        '#default_value' => $config->get('cc_auth'),
        '#placeholder' => $this->t('lowercase with _ and no spaces.'),
        //'#pattern' => '[a-z1-9_]+',  Haven't determined what strings are permissable.
        '#weight' => 3,
        '#disabled'=> isset($connected),
        '#required' => TRUE
      ]
    ];
    return parent::buildForm($form, $form_state);
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    $url = $form_state->getValue('trunkwards_node_url');
    $ccuser = $form_state->getValue('cc_user');
    $ccauth = $form_state->getValue('cc_auth');
    if (!mcapi_credcom_handshake($url, $ccuser, $ccauth)) {
      $form_state->setError($form['connection']['trunkwards_node_url'], $this->t('This node is not connected to %url. Check that the account exists and the name as password correspond.', ['%url' => $url]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->configFactory->getEditable('mcapi_credcom_spoof.settings')
      ->set('trunkwards_node_url', $form_state->getValue('trunkwards_node_url'))
      ->set('wallet_name', $form_state->getValue('wallet_name'))
      ->set('curr_id', $form_state->getValue('curr_id'))
      ->set('node_name', $form_state->getValue('node_name'))
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['mcapi_credcom_spoof.settings'];
  }

  private function defaultSiteName() : string {
    $site_name = $this->config('system.site')->get('name');
    return preg_replace('/[^a-z]/', '_', strtolower($site_name));
  }


}
