<?php

namespace Drupal\mcapi\Entity\Storage;

use Drupal\mcapi\Entity\Currency;
use Drupal\mcapi\Entity\Transaction;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Cache\Cache;

/**
 * Storage controller for Transaction entity. Must always handle Transaction
 * children, not passing them to the parent.
 */
class TransactionStorage extends TransactionIndexStorage {

  /**
   * {@inheritdoc}
   */
  public function loadBySerial($serial, $skip_access = FALSE) : Transaction {
    $transactions = $this->loadByProperties(['serial' => $serial]);
    return reset($transactions);
  }

  public function loadMultiple(array $xids = NULL) {
    // This works with xids, because views needs it.
    return parent::loadMultiple($xids);
  }

  /**
   * {@inheritdoc}
   *
   * Because the transaction entity is keyed by serial number not xid,
   * and because it contains child entities, we overwrite the whole save function.
   */
  public function save(EntityInterface $transaction) {
    // We can't start this process while another user might still be saving a
    // transaction because of the way the serial number is determined.
    if (empty($transaction->serial->value)) {
      $transaction->serial->value = $this->getSerial();
    }
    foreach ($transaction->flatten() as $entity) {
      $entity->serial->value = $transaction->serial->value;
      // Entity parent is 0 for the first entity and then the xid of the first
      // for all children saved consequently.
      $entity->parent->value = $parent??0;
      $return = parent::save($entity);
      if (!isset($parent)) {
        $parent = $entity->id();
      }
    }
    $transaction->enforceIsNew(FALSE);
    return $return;
  }

  /**
   * Get the next available serial number
   * @return int
   */
  private function getSerial() : int {
    $last = \Drupal::state()->get('mcapi.last_serial');
    if (!$last) { //TEMP
      $last = $this->database->query(
        "SELECT MAX([serial]) FROM {mcapi_transaction}"
      )->fetchField();
    }
    $next = $last+1;
    \Drupal::state()->set('mcapi.last_serial', $next);
    return $next;
  }

  /**
   * {@inheritdoc}
   */
  public function doSave($id, EntityInterface $entity) {
    // Children were already handled at self::save()
    $record = $this->mapToStorageRecord($entity);
    $record->changed = \Drupal::time()->getRequestTime();
    $return = parent::doSave($entity->xid->value, $entity);
    // The entity is no longer new.
    // because we were working on a clone.
    $entity->enforceIsNew(FALSE);

    return $return;
  }

  /**
   * {@inheritdoc}
   */
  public function resetCache(array $ids = NULL) {
    parent::resetCache($ids);
  }


  function delete(array $entities) {
    foreach($entities as $xid => $transaction) {
      foreach ($transaction->flatten() as $xid_c => $transaction) {
        $entities[$xid_c] = $transaction;
      }
    }
    parent::delete($entities);
  }

  /**
   * {@inheritdoc}
   */
  protected function doDelete($entities) {
    // First of all we need to get a flat array of all the entities.
    $transactions = [];
    foreach ($entities as $transaction) {
      $this->invalidateWalletCache($transaction);
      $transactions[$transaction->id()] = $transaction;
    }
    parent::doDelete($transactions);
  }

  /**
   * @param EntityInterface $entity
   * @param bool $update
   */
  protected function doPostSave(EntityInterface $entity, $update) {
    foreach ($entity->flatten() as $transaction) {
      $this->invalidateWalletCache($entity);
    }
    parent::doPostSave($entity, $update);
  }

  /**
   * @param Transaction $transaction
   */
  private function invalidateWalletCache(Transaction $transaction) {
    $payer_tags = $payee_tags = [];
    // in cases of migration rollbacks the wallet may not be present
    if ($payer_wallet = $transaction->payer->entity) {
      $payer_tags = $payer_wallet->getCacheTagsToInvalidate();
    }
    if ($payee_wallet = $transaction->payee->entity) {
      $payee_tags = $payee_wallet->getCacheTagsToInvalidate();
    }
    Cache::invalidateTags(array_merge($payer_tags, $payee_tags));
  }

}
